"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _DocsWorkspace_onDidChangeMarkdownDocument, _DocsWorkspace_onDidCreateMarkdownDocument, _DocsWorkspace_onDidDeleteMarkdownDocument, _MarkdownLinkComputer_instances, _MarkdownLinkComputer_getInlineLinks, _MarkdownLinkComputer_getReferenceLinks, _MarkdownLinkComputer_getLinkDefinitions;
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownLinkComputer = exports.DocsWorkspace = exports.MarkdownParser = void 0;
const fs = require("node:fs");
const path = require("node:path");
const glob = require("glob");
const MarkdownIt = require("markdown-it");
const vscode_markdown_languageservice_1 = require("@dsanders11/vscode-markdown-languageservice");
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_languageserver_textdocument_1 = require("vscode-languageserver-textdocument");
const vscode_uri_1 = require("vscode-uri");
// Helper function to work around import issues with ESM modules and ts-node
// eslint-disable-next-line no-new-func
const dynamicImport = new Function('specifier', 'return import(specifier)');
// Helper function from `vscode-markdown-languageservice` codebase
function tryDecodeUri(str) {
    try {
        return decodeURI(str);
    }
    catch (_a) {
        return str;
    }
}
// Helper function from `vscode-markdown-languageservice` codebase
function createHref(sourceDocUri, link, workspace) {
    if (/^[a-z-][a-z-]+:/i.test(link)) {
        // Looks like a uri
        return { kind: vscode_markdown_languageservice_1.HrefKind.External, uri: vscode_uri_1.URI.parse(tryDecodeUri(link)) };
    }
    const resolved = (0, vscode_markdown_languageservice_1.resolveInternalDocumentLink)(sourceDocUri, link, workspace);
    if (!resolved) {
        return undefined;
    }
    return {
        kind: vscode_markdown_languageservice_1.HrefKind.Internal,
        path: resolved.resource,
        fragment: resolved.linkFragment,
    };
}
function positionToRange(position) {
    return {
        start: {
            character: position.start.column - 1,
            line: position.start.line - 1,
        },
        end: { character: position.end.column - 1, line: position.end.line - 1 },
    };
}
const mdIt = MarkdownIt({ html: true });
class MarkdownParser {
    constructor() {
        this.slugifier = vscode_markdown_languageservice_1.githubSlugifier;
    }
    async tokenize(document) {
        return mdIt.parse(document.getText(), {});
    }
}
exports.MarkdownParser = MarkdownParser;
class DocsWorkspace {
    constructor(root, globs, ignoreGlobs = []) {
        //
        // These events are defined to fulfill the interface, but are never emitted
        // by this implementation since it's not meant for watching a workspace
        //
        _DocsWorkspace_onDidChangeMarkdownDocument.set(this, new vscode_languageserver_1.Emitter());
        this.onDidChangeMarkdownDocument = __classPrivateFieldGet(this, _DocsWorkspace_onDidChangeMarkdownDocument, "f").event;
        _DocsWorkspace_onDidCreateMarkdownDocument.set(this, new vscode_languageserver_1.Emitter());
        this.onDidCreateMarkdownDocument = __classPrivateFieldGet(this, _DocsWorkspace_onDidCreateMarkdownDocument, "f").event;
        _DocsWorkspace_onDidDeleteMarkdownDocument.set(this, new vscode_languageserver_1.Emitter());
        this.onDidDeleteMarkdownDocument = __classPrivateFieldGet(this, _DocsWorkspace_onDidDeleteMarkdownDocument, "f").event;
        this.documentCache = new Map();
        this.root = root;
        this.globs = globs;
        this.ignoreGlobs = ignoreGlobs;
    }
    get workspaceFolders() {
        return [vscode_uri_1.URI.file(this.root)];
    }
    async getAllMarkdownDocuments() {
        const files = this.globs.flatMap((pattern) => glob.sync(pattern, { ignore: this.ignoreGlobs, absolute: true, cwd: this.root }));
        for (const file of files) {
            const document = vscode_languageserver_textdocument_1.TextDocument.create(vscode_uri_1.URI.file(file).toString(), 'markdown', 1, fs.readFileSync(file, 'utf8'));
            this.documentCache.set(file, document);
        }
        return this.documentCache.values();
    }
    hasMarkdownDocument(resource) {
        const relativePath = path.relative(path.resolve(this.root), resource.fsPath);
        return (!relativePath.startsWith('..') &&
            !path.isAbsolute(relativePath) &&
            fs.existsSync(resource.fsPath));
    }
    async openMarkdownDocument(resource) {
        if (!this.documentCache.has(resource.fsPath)) {
            try {
                const document = vscode_languageserver_textdocument_1.TextDocument.create(resource.toString(), 'markdown', 1, fs.readFileSync(resource.fsPath, 'utf8'));
                this.documentCache.set(resource.fsPath, document);
            }
            catch (_a) {
                return undefined;
            }
        }
        return this.documentCache.get(resource.fsPath);
    }
    async stat(resource) {
        if (this.hasMarkdownDocument(resource)) {
            const stats = fs.statSync(resource.fsPath);
            return { isDirectory: stats.isDirectory() };
        }
        return undefined;
    }
    async readDirectory() {
        throw new Error('Not implemented');
    }
}
exports.DocsWorkspace = DocsWorkspace;
_DocsWorkspace_onDidChangeMarkdownDocument = new WeakMap(), _DocsWorkspace_onDidCreateMarkdownDocument = new WeakMap(), _DocsWorkspace_onDidDeleteMarkdownDocument = new WeakMap();
class MarkdownLinkComputer {
    constructor(workspace) {
        _MarkdownLinkComputer_instances.add(this);
        this.workspace = workspace;
    }
    async getAllLinks(document) {
        const { fromMarkdown } = (await dynamicImport('mdast-util-from-markdown'));
        const tree = fromMarkdown(document.getText());
        const links = [
            ...(await __classPrivateFieldGet(this, _MarkdownLinkComputer_instances, "m", _MarkdownLinkComputer_getInlineLinks).call(this, document, tree)),
            ...(await __classPrivateFieldGet(this, _MarkdownLinkComputer_instances, "m", _MarkdownLinkComputer_getReferenceLinks).call(this, document, tree)),
            ...(await __classPrivateFieldGet(this, _MarkdownLinkComputer_instances, "m", _MarkdownLinkComputer_getLinkDefinitions).call(this, document, tree)),
        ];
        return links;
    }
}
exports.MarkdownLinkComputer = MarkdownLinkComputer;
_MarkdownLinkComputer_instances = new WeakSet(), _MarkdownLinkComputer_getInlineLinks = async function _MarkdownLinkComputer_getInlineLinks(document, tree) {
    const { visit } = (await dynamicImport('unist-util-visit'));
    const documentUri = vscode_uri_1.URI.parse(document.uri);
    const links = [];
    visit(tree, (node) => node.type === 'link', (node) => {
        const link = node;
        const href = createHref(documentUri, link.url, this.workspace);
        if (href) {
            const range = positionToRange(link.position);
            // NOTE - These haven't been implemented properly, but their
            //        values aren't used for the link linting use-case
            const targetRange = range;
            const hrefRange = range;
            const fragmentRange = undefined;
            links.push({
                kind: vscode_markdown_languageservice_1.MdLinkKind.Link,
                href,
                source: {
                    hrefText: link.url,
                    resource: documentUri,
                    range,
                    targetRange,
                    hrefRange,
                    fragmentRange,
                    pathText: link.url.split('#')[0],
                },
            });
        }
    });
    return links;
}, _MarkdownLinkComputer_getReferenceLinks = async function _MarkdownLinkComputer_getReferenceLinks(document, tree) {
    const { visit } = (await dynamicImport('unist-util-visit'));
    const links = [];
    visit(tree, (node) => ['imageReference', 'linkReference'].includes(node.type), (node) => {
        const link = node;
        const range = positionToRange(link.position);
        // NOTE - These haven't been implemented properly, but their
        //        values aren't used for the link linting use-case
        const targetRange = range;
        const hrefRange = range;
        links.push({
            kind: vscode_markdown_languageservice_1.MdLinkKind.Link,
            href: {
                kind: vscode_markdown_languageservice_1.HrefKind.Reference,
                ref: link.label,
            },
            source: {
                hrefText: link.label,
                resource: vscode_uri_1.URI.parse(document.uri),
                range,
                targetRange,
                hrefRange,
                fragmentRange: undefined,
                pathText: link.label,
            },
        });
    });
    return links;
}, _MarkdownLinkComputer_getLinkDefinitions = async function _MarkdownLinkComputer_getLinkDefinitions(document, tree) {
    const { visit } = (await dynamicImport('unist-util-visit'));
    const documentUri = vscode_uri_1.URI.parse(document.uri);
    const links = [];
    visit(tree, (node) => node.type === 'definition', (node) => {
        const definition = node;
        const href = createHref(documentUri, definition.url, this.workspace);
        if (href) {
            const range = positionToRange(definition.position);
            // NOTE - These haven't been implemented properly, but their
            //        values aren't used for the link linting use-case
            const targetRange = range;
            const hrefRange = range;
            const fragmentRange = undefined;
            links.push({
                kind: vscode_markdown_languageservice_1.MdLinkKind.Definition,
                href,
                ref: {
                    range,
                    text: definition.label,
                },
                source: {
                    hrefText: definition.url,
                    resource: documentUri,
                    range,
                    targetRange,
                    hrefRange,
                    fragmentRange,
                    pathText: definition.url.split('#')[0],
                },
            });
        }
    });
    return links;
};
//# sourceMappingURL=markdown.js.map